# Generated by Django 2.0.3 on 2019-03-28 21:01

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='LastScheduledFor',
            fields=[
                ('task', models.CharField(max_length=255, primary_key=True, serialize=False)),
                ('execute_at', models.DateTimeField()),
            ],
        ),
    ]
