# Generated by Django 3.2.5 on 2021-07-16 19:58

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('dpq', '0002_auto_20190419_2057'),
    ]

    if hasattr(models, 'JSONField'):
        operations = [
            migrations.AlterField(
                model_name='job',
                name='args',
                field=models.JSONField(),
            ),
        ]
    else:
        operations = []
